"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.PrivateChannel = void 0;
const axios_1 = require("axios");
const log_1 = require("../log");
const https = require("https");
const bottleneck_1 = require("bottleneck");
const httpsAgent = new https.Agent({
    rejectUnauthorized: false,
});
class PrivateChannel {
    constructor(options) {
        this.options = options;
        this.limiter = new bottleneck_1.default({
            maxConcurrent: options.maxConcurrentAuthRequests
        });
    }
    authenticate(socket, data) {
        let authHost;
        if (this.options.multiTenant && data.auth && data.auth.host) {
            try {
                const customHost = new URL(data.auth.host);
                authHost = `${customHost.protocol}//${customHost.host}`;
            }
            catch (e) {
                log_1.Log.warning(`[${new Date().toISOString()}] - Invalid custom host provided: ${data.auth.host}`);
                authHost = this.authHost(socket);
            }
        }
        else {
            authHost = this.authHost(socket);
        }
        if (this.options.allowSubDomains) {
            const referer = new URL(data.auth.headers.referer || '');
            const refererDomain = this.extractParentDomain(referer.hostname);
            const authDomain = this.extractParentDomain(new URL(this.authHost(socket)).hostname);
            if (refererDomain === authDomain) {
                authHost = `${referer.protocol}//${referer.host}`;
            }
        }
        let options = {
            url: authHost + this.options.authEndpoint,
            form: { channel_name: data.channel },
            headers: (data.auth && data.auth.headers) ? data.auth.headers : {},
            allowSelfSigned: this.options.allowSelfSigned || false,
        };
        if (this.options.devMode) {
            log_1.Log.info(`[${new Date().toISOString()}] - Sending auth request to: ${options.url}\n`);
        }
        return this.limiter.schedule(() => this.serverRequest(socket, options));
    }
    authHost(socket) {
        let authHosts = (this.options.authHost) ?
            this.options.authHost : this.options.host;
        if (typeof authHosts === "string") {
            authHosts = [authHosts];
        }
        let authHostSelected = authHosts[0] || 'http://localhost';
        if (socket.request.headers.referer) {
            let referer = new URL(socket.request.headers.referer);
            for (let authHost of authHosts) {
                authHostSelected = authHost;
                if (this.hasMatchingHost(referer, authHost)) {
                    authHostSelected = `${referer.protocol}//${referer.host}`;
                    break;
                }
            }
            ;
        }
        if (this.options.devMode) {
            log_1.Log.error(`[${new Date().toISOString()}] - Preparing authentication request to: ${authHostSelected}`);
        }
        return authHostSelected;
    }
    hasMatchingHost(referer, host) {
        return (referer.hostname && referer.hostname.substr(referer.hostname.indexOf('.')) === host) ||
            `${referer.protocol}//${referer.host}` === host ||
            referer.host === host;
    }
    serverRequest(socket, options) {
        return new Promise((resolve, reject) => {
            options.headers = this.prepareHeaders(socket, options);
            const agent = options.allowSelfSigned
                ? new https.Agent({ rejectUnauthorized: false })
                : undefined;
            axios_1.default.post(options.url, options.form, {
                headers: options.headers,
                httpsAgent: agent,
            }).then((response) => {
                if (this.options.devMode) {
                    log_1.Log.info(`[${new Date().toISOString()}] - ${socket.id} authenticated for: ${options.form.channel_name}`);
                }
                resolve(response.data);
            })
                .catch((error) => {
                if (error.response) {
                    log_1.Log.warning(`[${new Date().toISOString()}] - ${socket.id} could not be authenticated to ${options.form.channel_name}`);
                    log_1.Log.error(error.response.data || error.message);
                    reject({ reason: 'Client cannot be authenticated.', status: error.response.status });
                }
                else {
                    log_1.Log.error(`[${new Date().toISOString()}] - Error sending authentication request: ${error.message}`);
                    reject({ reason: 'Error sending authentication request.', status: 0 });
                }
            });
        });
    }
    prepareHeaders(socket, options) {
        options.headers['Cookie'] = options.headers['Cookie'] || socket.request.headers.cookie;
        options.headers['X-Requested-With'] = 'XMLHttpRequest';
        return options.headers;
    }
    extractParentDomain(hostname) {
        const parts = hostname.split('.');
        if (parts.length > 2) {
            return parts.slice(-2).join('.');
        }
        return hostname;
    }
}
exports.PrivateChannel = PrivateChannel;
